/*******************************************************************************
 * Copyright 2022 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef MODALCV_OPT_FLOW_H
#define MODALCV_OPT_FLOW_H


#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>


////////////////////////////////////////////////////////////////////////////////
// Semi-Dense Optical Flow on CVP
////////////////////////////////////////////////////////////////////////////////

typedef enum mcv_opt_flow_mode_t{
    MCV_OPT_FLOW_ONE_PASS = 0,
    MCV_OPT_FLOW_SEVEN_PASS
}mcv_opt_flow_mode_t;

typedef struct mcv_opt_flow_config_t{
    int width;
    int height;
    mcv_opt_flow_mode_t mode;
    int en_stats;
}mcv_opt_flow_config_t;

typedef struct mcv_motion_vector{
    int32_t mv_x; // signed motion vector for an 8x8 block, horizontal
    int32_t mv_y; // signed motion vector for an 8x8 block, vertical
    int32_t conf;
} mcv_motion_vector;

typedef struct mcv_opt_flow_stats{
    uint16_t variance;
    uint8_t mean;
    uint8_t reserved;
    uint16_t best_SAD;
    uint16_t SAD_0; 
} mcv_opt_flow_stats;

typedef struct mcv_opt_flow_output{
    mcv_motion_vector* motion_vecs;
    mcv_opt_flow_stats* opt_flow_stats;
    int n_blocks;                           // if i understand correctly, we will have 1 motion vec and stat output per block
} mcv_opt_flow_output;


int mcv_opt_flow_init(mcv_opt_flow_config_t config, mcv_opt_flow_output* opt_flow_output);
int mcv_opt_flow_deinit(void);

// this func will allocate memory in your mcv_opt_flow_output struct 
int mcv_opt_flow_process(uint8_t* ref_img, uint8_t* curr_img, int is_new_ref_img, int is_new_curr_img, mcv_opt_flow_output* of_output);

#ifdef __cplusplus
}
#endif

#endif // MODALCV_OPT_FLOW_H