/*******************************************************************************
 * Copyright 2022 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef MODALCV_PYR_FPX_H
#define MODALCV_PYR_FPX_H

#include <stdint.h>
#include "pyr.h"
#include "fpx.h"

#ifdef __cplusplus
extern "C" {
#endif


////////////////////////////////////////////////////////////////////////////////
// Image Pyramid and Feature Point Extraction on CVP
////////////////////////////////////////////////////////////////////////////////

typedef struct mcv_fpx_pyr_config_t {
	int width;											// input width
	int height;											// input height
	uint32_t n_octaves;									// Number of octaves. Image size is halved between adjacent octaves
	uint32_t n_scales_per_octave;						// Number of scales per octave, including the octave. Number of levels is equal to nScalesPerOctave * nOctaves.
	uint32_t n_filter_coeffs[MCV_MAX_PYRAMID_OCTAVES];	// Array of divide-by-2 filter coefficients. Range: 0 (sharpest) through 4 (flattest)
	mcv_fpx_mode_t mode;
	mcv_fpx_nms_t nms_mode;
	uint32_t score_threshold;
	uint8_t robustness;                                 // 0-127, default 10
	int enable_fpx[MCV_MAX_PYRAMID_OCTAVES];           // flag of whether or not to extract at this level
} mcv_fpx_pyr_config_t;


typedef struct mcv_fpx_pyr {
	int n_levels;										// Number of levels is equal to nScalesPerOctave * nOctaves
	int width[MCV_MAX_PYRAMID_LEVELS];					// width of image per level
	int height[MCV_MAX_PYRAMID_LEVELS];					// height of image per level
	uint8_t* image_pyramids[MCV_MAX_PYRAMID_LEVELS];	// array of image pyramids, malloced by mcv_pyr_init
	mcv_fpx_feature_t* fpx[MCV_MAX_PYRAMID_LEVELS];     // feature points per level
} mcv_fpx_pyr;


// mcv_pyr_fpx_init will allocate memory in the mcv_pyramid output, amd return required buf size for features (user allocated, per level)
int mcv_pyr_fpx_init(mcv_fpx_pyr_config_t config, mcv_fpx_pyr* fpx_pyr, int required_buf_size[MCV_MAX_PYRAMID_OCTAVES]);

int mcv_pyr_fpx_change_zone_dims(uint32_t x, uint32_t y, uint8_t level);

// extracts pyramid, fills the mcv_pyramid data as well as all feature point information
int mcv_pyr_fpx_extract(uint8_t* img, mcv_fpx_pyr* fpx_pyr, uint32_t* max_score, int* n_points, uint8_t cam);

// cleanup
int mcv_pyr_fpx_deinit(void);


#ifdef __cplusplus
}
#endif

#endif // end #define MODALCV_PYR_FPX_H
